# STL Container: `std::map`

`std::map` is an associative container in the C++ Standard Template Library (STL) that stores key-value pairs in a sorted order based on the keys. It is typically implemented as a balanced binary search tree (usually a red-black tree), providing logarithmic time complexity for insertion, deletion, and search operations.

## Key Characteristics

- Stores key-value pairs (elements)
- Keys are unique and sorted
- Provides logarithmic time complexity (O(log n)) for insertion, deletion, and search operations
- Allows direct access to values using keys (like an associative array)
- Supports bidirectional iterators
- Allows for custom key comparison functions
- Suitable for scenarios where fast key-based lookup and ordered traversal are required

## Example 1: Basic Usage

```cpp
#include <iostream>
#include <map>
#include <string>

int main() {
    std::map<std::string, int> ages;

    // Inserting elements
    ages["Alice"] = 30;
    ages.insert({"Bob", 25});
    ages.insert(std::make_pair("Charlie", 35));

    // Accessing elements
    std::cout << "Alice's age: " << ages["Alice"] << std::endl;

    // Checking if a key exists
    if (ages.find("David") == ages.end()) {
        std::cout << "David is not in the map" << std::endl;
    }

    // Iterating through the map
    for (const auto& pair : ages) {
        std::cout << pair.first << " is " << pair.second << " years old" << std::endl;
    }

    // Size of the map
    std::cout << "Number of entries: " << ages.size() << std::endl;

    // Removing an element
    ages.erase("Bob");

    return 0;
}
```

### Explanation:

- A `map` with string keys and integer values is created
- Elements are inserted using different methods: `[]` operator, `insert()`, and `std::make_pair`
- The `[]` operator is used for direct access to values
- `find()` is used to check if a key exists
- The map is iterated using a range-based for loop, showing key-value pairs
- `size()` returns the number of elements
- `erase()` removes an element by its key

## Example 2: Custom Comparator for Descending Order

```cpp
#include <iostream>
#include <map>
#include <string>

// Custom comparator for descending order of keys
struct DescendingOrder {
    bool operator()(const std::string& a, const std::string& b) const {
        return a > b;
    }
};

int main() {
    std::map<std::string, int, DescendingOrder> scores;

    scores["Alice"] = 95;
    scores["Bob"] = 89;
    scores["Charlie"] = 92;
    scores["David"] = 78;

    for (const auto& pair : scores) {
        std::cout << pair.first << ": " << pair.second << std::endl;
    }

    return 0;
}
```

### Explanation:

- A custom comparator `DescendingOrder` is defined for string keys
- The `map` is initialized with this custom comparator
- Elements are automatically sorted in descending order of keys
- This example demonstrates how to use a custom sorting criteria with `std::map`

## Example 3: Using `map` with Custom Objects

```cpp
#include <iostream>
#include <map>
#include <string>

class Student {
public:
    Student(const std::string& name, int age) : name(name), age(age) {}

    std::string getName() const { return name; }
    int getAge() const { return age; }

private:
    std::string name;
    int age;
};

// Custom comparator for Student objects
struct StudentComparator {
    bool operator()(const Student& a, const Student& b) const {
        return a.getName() < b.getName();
    }
};

int main() {
    std::map<Student, double, StudentComparator> studentGrades;

    studentGrades[Student("Alice", 20)] = 3.8;
    studentGrades[Student("Bob", 22)] = 3.5;
    studentGrades[Student("Charlie", 21)] = 3.9;

    for (const auto& pair : studentGrades) {
        std::cout << pair.first.getName() << " (Age: " << pair.first.getAge() 
                  << "): GPA = " << pair.second << std::endl;
    }

    return 0;
}
```

### Explanation:

- A `Student` class is defined with name and age attributes
- A custom comparator `StudentComparator` is created to sort `Student` objects by name
- The `map` uses `Student` objects as keys and `double` as values
- This example shows how to use custom objects as keys in a `map`

## Example 4: Nested Maps and Complex Data Structures

```cpp
#include <iostream>
#include <map>
#include <string>
#include <vector>

// Function to print the course roster
void printCourseRoster(const std::map<std::string, std::map<std::string, std::vector<int>>>& university) {
    for (const auto& dept : university) {
        std::cout << "Department: " << dept.first << std::endl;
        for (const auto& course : dept.second) {
            std::cout << "  Course: " << course.first << ", Students: ";
            for (int studentId : course.second) {
                std::cout << studentId << " ";
            }
            std::cout << std::endl;
        }
        std::cout << std::endl;
    }
}

int main() {
    // Map structure: Department -> (Course -> List of Student IDs)
    std::map<std::string, std::map<std::string, std::vector<int>>> university;

    // Adding data
    university["Computer Science"]["Programming 101"] = {1001, 1002, 1003};
    university["Computer Science"]["Data Structures"] = {1002, 1004, 1005};
    university["Mathematics"]["Calculus I"] = {1001, 1005, 1006};
    university["Mathematics"]["Linear Algebra"] = {1003, 1004, 1006};

    // Print the course roster
    printCourseRoster(university);

    // Adding a new student to a course
    university["Computer Science"]["Programming 101"].push_back(1007);

    // Checking if a department exists
    std::string deptToCheck = "Physics";
    if (university.find(deptToCheck) == university.end()) {
        std::cout << deptToCheck << " department does not exist." << std::endl;
    }

    return 0;
}
```

### Explanation:

- This example demonstrates the use of nested maps to represent a complex data structure
- The outer map uses department names as keys and inner maps as values
- The inner maps use course names as keys and vectors of student IDs as values
- The `printCourseRoster` function shows how to iterate through this nested structure
- New data can be easily added to the structure at any level
- The `find` method is used to check for the existence of a key (department in this case)

## Additional Considerations

1. **Performance**: While `map` provides good performance for most operations, consider using `std::unordered_map` for faster lookup times if sorting is not required.

2. **Memory Usage**: `map` typically uses more memory than a simple array due to its tree structure.

3. **Key Immutability**: Once a key-value pair is inserted, the key cannot be modified. To change a key, you must remove the old pair and insert a new one.

4. **Value Modification**: Values can be modified directly through iterators or using the `[]` operator.

5. **Default-Constructed Values**: Using the `[]` operator on a non-existent key will insert a default-constructed value.

## Summary

`std::map` is a versatile container in C++ for storing key-value pairs in a sorted order. Key points to remember:

- It maintains key-value pairs sorted by keys
- Provides logarithmic time complexity for insertion, deletion, and search operations
- Keys are unique; inserting a duplicate key will not change the map
- Supports custom comparators for key ordering
- Useful for scenarios requiring fast key-based lookup and ordered traversal
- Can be used with both built-in and custom types as keys and values
- Suitable for representing hierarchical or nested data structures

`std::map` is particularly useful in situations where you need to:
- Implement dictionaries or lookup tables
- Maintain sorted associations between keys and values
- Represent hierarchical data
- Perform range-based queries on keys

Understanding when to use `std::map` versus other containers like `std::unordered_map` or `std::vector` is crucial for writing efficient and clear C++ code in various application domains. Its ordered nature makes it ideal for scenarios where both fast lookup and sorted traversal are important.

## Related

- What are the key differences between std::map and std::set
- How does std::map handle key-value pairs
- What are the time complexities for common operations in std::map
- Can you explain the insertion process in std::map
- How does std::map maintain the order of elements
